  
"
    __________________________________________________
    @Alejandro F. Reimondo 18/10/02
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"!

Object subclass: #ANSIStandardObject
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''    !

ANSIStandardObject subclass: #BehavioralDescription
  instanceVariableNames: 
    ' protocol text '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #BehavioralDescription) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1.2.1 Behavioral Description

The behavior of a message is described with English text. Basic operations are described in terms of their effects on the abstract state of the object.'    !

ANSIStandardObject subclass: #MessageSpecification
  instanceVariableNames: 
    ' selector synopsis definition refinement parameterSpecifications returnValueSpecifications errors message '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #MessageSpecification) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1.2 Message Specification

A message specification describes an individual message in the context of a particular protocol. It is defined by a message selector, a behavioral description, a set of parameter specifications, and a set of return value specifications.


'   !

ANSIStandardObject subclass: #ProtocolGroupSpecification
  instanceVariableNames: 
    ' name description protocols '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ProtocolGroupSpecification) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1.4.4 Protocol Groupings

Within grouping, protocols are ordered according to their conformance lattice. Secondary sorting is alphabetical by protocol name.

Some groups:

5.3 Fundamentals Protocols

      <Object class> isConformTo: <instantiator> isConformTo: <Object>

5.6 Numeric Protocols

     <Integer> isConformTo: <rational> isConformTo: <Number> isConformTo: <Magnitude>
'  !

ANSIStandardObject subclass: #ProtocolSpecification
  instanceVariableNames: 
    ' name messageSpecifications description conformsTo '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ProtocolSpecification) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1 Protocols

Protocols are used in the specification both to define the behavior of concrete classes, and also to factor common behavior from other procotols through a relationship called conformance.

Protocols that define behavior of objects bound to global names are "concrete protocols".

Protocols that are used only as a factoring mechanism are referred to as "abstract protocols".'  !

ANSIStandardObject subclass: #StandardClassLibrary
  instanceVariableNames: 
    ' protocolGroups '
  classVariableNames: ''
  poolDictionaries: ''    !

StandardClassLibrary class instanceVariableNames: ' default '  !

ANSIStandardObject subclass: #ValueSpecification
  instanceVariableNames: 
    ' protocols aliasing '
  classVariableNames: ''
  poolDictionaries: ''  !

ValueSpecification subclass: #ParameterSpecification
  instanceVariableNames: 
    ' name '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ParameterSpecification) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1.2.2 Parameters Specification

A parameter specification is defined by a parameter name, a parameter interface definition, and a parameter aliasing attribute.

A parameter specification places constraints on the parameter in terms of protocol conformance, and provides information concerning how the parameter is used by implementations of the message.

The parameter specification is prescriptive in that it states requirements to which the parameters must conform. A procotol makes a conformance "requirement" statement about parameters.'    !

ValueSpecification subclass: #ReturnValueSpecification
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ReturnValueSpecification) comment: '#addedByClau.
ANSI Capitulo 5. Standart Class Library

5.1.2.3 Return Value Specification

The return value information is descriptive in that it provides information about the result being returned. A protocol makes a conformance "commitment" concerning the return value.

The specifications guarantees that the return value will conform to the specified protocol.'  !


!ANSIStandardObject class methods !
  
from: anObject
	"Returns an instance of the receiver."

	^self new initialize: anObject! !



!ANSIStandardObject methods !
  
initialize: anObject
	" Private - Initialize the receiver. "

	^self implementedBySubclass! !



!BehavioralDescription methods !

protocol
    "Returns the instance var of the receiver
     for external manipulation"
    ^protocol!

protocol: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    protocol := anObject!
   
text
    "Returns the instance var of the receiver
     for external manipulation"
    ^text!

text: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    text := anObject! !



!MessageSpecification methods !
  
clean: aString
	"Answer a String without comment characters"

	^aString reject: [ :char | char = $" ]!
   
definition
    "Returns the instance var of the receiver
     for external manipulation"
    ^definition!

definition: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    definition := anObject!
   
errors
    "Returns the instance var of the receiver
     for external manipulation"
    ^errors!

errors: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    errors := anObject!
   
hasParameters
	"Private - Answer true if the receiver has parameters"

	^(Message numberOfArgumentsFor: self selector) ~= 0!
 
initialize: aChapter
	" Private - Initialize the receiver. "

	self
		readSelectorFrom: (ReadStream on: aChapter title);
		readContentsFrom: (ReadStream on: aChapter contents contents) in: aChapter.!

initializeParameterSpecifications
	" Private - Initializes the receiver's parameterSpecifications"

	parameterSpecifications := OrderedCollection new.!
  
initializeReturnValueSpecifications
	" Private - Initializes the receiver's returnValueSpecifications"

	returnValueSpecifications := OrderedCollection new.!

message
    "Returns the instance var of the receiver
     for external manipulation"
    ^message!
  
message: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    message := anObject!
 
parameterSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	parameterSpecifications isNil ifTrue: [ self initializeParameterSpecifications ].
    ^parameterSpecifications!

printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	self printSourceOn: aStream!
   
printParametersOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies
	the receiver's parameterSpecification"

	aStream nextPutAll: 'Parameters:'; cr.
	self parameterSpecifications
						do: [:each | aStream nextPutAll: each; cr]!
   
printReturnValuesOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies
	the receiver's parameterSpecification"

	aStream nextPutAll: 'Return Values:'; cr.
	self returnValueSpecifications
						do: [:each | aStream nextPutAll: each; cr]!

printSourceOn: aStream
		"Append to aStream the source code for the receiver"

		aStream
        	nextPutAll: (self clean: self message); cr;
        	nextPutAll: '"Comment from NCITS-319...';cr;
        	nextPutAll: 'Synopsis:'; cr;
        	nextPutAll: (self clean: self synopsis); cr;
        	nextPutAll: (self clean: self definition); cr.
        self printParametersOn: aStream;
        	   printReturnValuesOn: aStream.
        aStream cr;
			nextPutAll: 'Errors:'; cr;
        	nextPutAll: (self clean: self errors); nextPut: $"; cr; tab;
        	nextPutAll: 'self notImplementedYet'
!
   
readContentsFrom: aStream in: aChapter
	" Private - Read the receiver's contents from aStream. "

	self
		readSynopsisFrom: aStream in: aChapter;
		readDefinitionFrom: aStream in: aChapter;
		readParametersFrom: aStream in: aChapter;
		readReturnValuesFrom: aStream in: aChapter;
		readErrorsFrom: aStream in: aChapter;
		yourself!

readDefinitionFrom: aStream in: aChapter
	" Private - Read the receiver's definition from aStream. "

	self hasParameters
				ifTrue: [^self definition: (aChapter skipToLine: 'Parameters' in: aStream)].

	self definition: (aChapter skipToLineStartingWith: 'Return' in: aStream)!

readErrorsFrom: aStream in: aChapter
	" Private - Read the receiver's errors from aStream. "

	aChapter upToLine: 'Errors' in: aStream.
	self errors: (aStream upTo: nil).!
 
readParametersFrom: aStream in: aChapter
	" Private - Read the receiver's parameters from aStream. "
	
	| text |
	self hasParameters ifFalse: [ ^self ].
	aChapter upToLine: 'Parameters' in: aStream.
	text := ReadStream on: (aChapter skipToLineStartingWith: 'Return' in: aStream).
	[text atEnd] whileFalse: [
		self parameterSpecifications add: text nextLine
			"add: (ParameterSpecification from: (ReadStream on: text nextLine))"
	].!

readReturnValuesFrom: aStream in: aChapter
	" Private - Read the receiver's returnValue from aStream. "

	| text |
	#Gallego.
	aChapter upToLineStartingWith: 'Return' in: aStream.
	text := ReadStream on: (aChapter skipToLine: 'Errors' in: aStream).
	[text atEnd] whileFalse: [
		self returnValueSpecifications add: text nextLine
			"add: (ReturnValueSpecification from: (ReadStream on: text nextLine))"
	].!
   
readSelectorFrom: aStream
	" Private - Read the receiver's selector from aStream. "

	| elements |
	message := aStream upTo: $:; countBlanks; upTo: nil.
	selector := (message includes: $:)
	ifTrue: [
		elements := String new.
		(message asArrayOfSubstrings
			select: [:each| each includes: $: ])
			do: [:one| elements := elements , one ].
		elements asSymbol
	]
	ifFalse: [ (message upTo: $ ) asSymbol ]!
 
readSynopsisFrom: aStream in: aChapter
	" Private - Read the receiver's synopsis from aStream. "

	aChapter upToLine: 'Synopsis' in: aStream.
	self synopsis: (aChapter skipToLineStartingWith: 'Definition:' in: aStream).!

refinement
    "Returns the instance var of the receiver
     for external manipulation"
    ^refinement!

refinement: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    refinement := anObject!
   
returnValueSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	returnValueSpecifications isNil ifTrue: [ self initializeReturnValueSpecifications ].
    ^returnValueSpecifications!

selector
    "Returns the instance var of the receiver
     for external manipulation"
    ^selector!

selector: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    selector := anObject!
   
synopsis
    "Returns the instance var of the receiver
     for external manipulation"
    ^synopsis!

synopsis: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    synopsis := anObject! !



!ProtocolGroupSpecification class methods !
  
name: aString
    "Return a new instance of the receiver named aString"

     ^self new
        name: aString;
        yourself.
        ! !



!ProtocolGroupSpecification methods !
 
description
    "Returns the instance var of the receiver
     for external manipulation"
    ^description!
  
description: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    description := anObject!
 
initialize: aChapter
    "Private - Initialize the receiver."

	| stream |
	stream := ReadStream on: aChapter contents asArray.
    self
		name: aChapter title;
		description: stream next contents contents;
		yourself.
	[stream atEnd] whileFalse: [
		self protocols add: (self protocolFrom: stream next)
	].!
 
initializeProtocols
	" Private - Initializes the receiver's protocols"

	protocols := OrderedCollection new.!

name
    "Returns the instance var of the receiver
     for external manipulation"
    ^name!

name: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    name := anObject!
   
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	aStream
		nextPutAll: self name; cr; cr;
		nextPutAll: self description!
 
protocolFrom: aChapter
	" Private - Returns a protocol from aChapter. "

	^ProtocolSpecification from: aChapter!
 
protocols
    "Returns the instance var of the receiver
     for external manipulation"

	protocols isNil ifTrue: [ self initializeProtocols ].
    ^protocols! !



!ProtocolSpecification methods !
  
compileInClass: aClass
	"Compile all the receiver's messageSpecifications in aClass"

	| assoc |
	self messageSpecifications do: [:msg |
														assoc := aClass compile: msg printString notifying: Transcript.
														assoc notNil
																	ifTrue: [SourceManager current
																					logSource: msg printString
							        												forSelector: assoc key
							        												inClass: aClass ]
														]!

conformsTo
    "Returns the instance var of the receiver
     for external manipulation"
    ^conformsTo!

conformsTo: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    conformsTo := anObject!
   
description
    "Returns the description of the receiver."

    ^description
!
  
description: aString
    "Sets the description of the receiver."

    description := aString

!

initialize: aChapter
	" Private - Initialize the receiver. "

	| stream |
	stream := ReadStream on: aChapter title.
    self name: (stream upTo: $<;upTo: $>).
	stream := ReadStream on: aChapter contents asArray.
	self
		readContentsFrom: (ReadStream on: stream next text)
		in: aChapter.
	[stream atEnd] whileFalse: [
		self messageSpecifications add: (self messageSpecificationFrom: stream next)
	].!
   
initializeMessageSpecifications
	" Private - Initializes the receiver's messageSpecifications"

	messageSpecifications := OrderedCollection new!
 
messageSpecificationFrom: aChapter
	" Private - Returns the next messageSpecification from aChapter. "

	^MessageSpecification from: aChapter!
   
messageSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	messageSpecifications isNil ifTrue: [ self initializeMessageSpecifications ].
    ^messageSpecifications!

name
    "Returns the name of the receiver."

    ^name
!
   
name: aString
    "Sets the name of the receiver."

    name := aString

!
 
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	aStream
		nextPutAll: 'Protocol: <';
		nextPutAll: self name;
		nextPut: $>;cr;cr;
		nextPutAll: 'Conforms To:'; cr.
	self conformsTo
		do: [ :each | each printOn: aStream	].
	aStream
		cr;cr;
		nextPutAll: self description!
  
readConformsFrom: aStream in: aChapter
	" Private - Read the receiver's conformsTo from aStream. "

	| conforms |
	aChapter upToLine: 'Conforms To' in: aStream.
	conforms := ReadStream on: (aChapter skipToLine: 'Description' in: aStream).
	conformsTo := Set new.
	[conforms atEnd] whileFalse: [
		conformsTo add: (conforms upTo: $,) trimBlanks
	].!
   
readContentsFrom: aStream in: aChapter
	" Private - Read the receiver's contents from aStream. "

	self
		readConformsFrom: aStream in: aChapter;
		readDescriptionFrom: aStream in: aChapter;
		yourself!

readDescriptionFrom: aStream in: aChapter
	" Private - Read the receiver's description from aStream. "

	self description: (aStream upTo: nil)! !



!StandardClassLibrary class methods !
   
default
    "Returns the instance var of the receiver
     for external manipulation"

	default isNil ifTrue: [ self initializeDefault ].
    ^default!

default: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    default := anObject!
 
initializeDefault
	" Private - Initializes the receiver's default"

	default := self from: (Smalltalk at: #NCITSDocument) default.!
  
reset
	" Private - Reset the receiver. "

	self default: nil.! !



!StandardClassLibrary methods !
  
defaultGroupIDs
	" Private - Returns the IDs to be used to read the receiver's groups. "

	^(3 to: 10) collect: [:each| '5.',each asString ]!

groups:  ids from: aDocument
	" Private - Returns the groups read from aDocument with ids. "

	^ids collect: [:id| ProtocolGroupSpecification from: aDocument @ id ]!

initialize: aDocument
	" Private - Initialize the receiver. "

	self protocolGroups addAll: (self groups: self defaultGroupIDs from: aDocument)!
 
initializeProtocolGroups
	" Private - Initializes the receiver's protocolGroups"

	protocolGroups := OrderedCollection new.!
 
protocolGroups
    "Returns the instance var of the receiver
     for external manipulation"

	protocolGroups isNil ifTrue: [ self initializeProtocolGroups ].
    ^protocolGroups!

protocolNamed: aName
	"Return the protocol named aName or generates an error if absent."

	^self protocolNamed: aName ifAbsent: [ self error: 'Element not found.' ]!

protocolNamed: aName ifAbsent: aBlock
	"Return the protocol named aName or evaluates aBlock if absent."

	| protocol |
	self protocolGroups do: [:each|
		protocol := each protocols
			detect: [:one| one name = aName ]
			ifNone: [].
		protocol notNil ifTrue: [ ^protocol ].
	].
	^aBlock value! !



!ValueSpecification methods !
  
aliasing
    "Returns the instance var of the receiver
     for external manipulation"
    ^aliasing!

aliasing: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    aliasing := anObject!
   
initializeProtocols
	" Private - Initializes the receiver's protocols"

	protocols := OrderedCollection new.!

protocols
    "Returns the instance var of the receiver
     for external manipulation"

	protocols isNil ifTrue: [ self initializeProtocols ].
    ^protocols! !



!ParameterSpecification class methods !
   
name: aString
    "Return a new instance of the receiver named aString"

     ^self new
        name: aString;
        yourself.! !



!ParameterSpecification methods !
   
name
    "Returns the instance var of the receiver
     for external manipulation"
    ^name!

name: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    name := anObject! !
