18/10/02 04:38:28 p.m.
  
from: anObject
	"Returns an instance of the receiver."

	^self new initialize: anObject!   
protocol
    "Returns the instance var of the receiver
     for external manipulation"
    ^protocol!  
text: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    text := anObject! 
text
    "Returns the instance var of the receiver
     for external manipulation"
    ^text!  
protocol: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    protocol := anObject! 
initialize: anObject
	" Private - Initialize the receiver. "

	^self implementedBySubclass!
name: aString
    "Return a new instance of the receiver named aString"

     ^self new
        name: aString;
        yourself.
        !  
default
    "Returns the instance var of the receiver
     for external manipulation"

	default isNil ifTrue: [ self initializeDefault ].
    ^default!  
default: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    default := anObject!   
initializeDefault
	" Private - Initializes the receiver's default"

	default := self from: (Smalltalk at: #NCITSDocument) default.!
reset
	" Private - Reset the receiver. "

	self default: nil.! 
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	self printSourceOn: aStream! 
message
    "Returns the instance var of the receiver
     for external manipulation"
    ^message!
readContentsFrom: aStream in: aChapter
	" Private - Read the receiver's contents from aStream. "

	self
		readSynopsisFrom: aStream in: aChapter;
		readDefinitionFrom: aStream in: aChapter;
		readParametersFrom: aStream in: aChapter;
		readReturnValuesFrom: aStream in: aChapter;
		readErrorsFrom: aStream in: aChapter;
		yourself!  
refinement
    "Returns the instance var of the receiver
     for external manipulation"
    ^refinement!  
definition
    "Returns the instance var of the receiver
     for external manipulation"
    ^definition!  
initialize: aChapter
	" Private - Initialize the receiver. "

	self
		readSelectorFrom: (ReadStream on: aChapter title);
		readContentsFrom: (ReadStream on: aChapter contents contents) in: aChapter.!  
message: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    message := anObject!   
printSourceOn: aStream
		"Append to aStream the source code for the receiver"

		aStream
        	nextPutAll: (self clean: self message); cr;
        	nextPutAll: '"Comment from NCITS-319...';cr;
        	nextPutAll: 'Synopsis:'; cr;
        	nextPutAll: (self clean: self synopsis); cr;
        	nextPutAll: (self clean: self definition); cr.
        self printParametersOn: aStream;
        	   printReturnValuesOn: aStream.
        aStream cr;
			nextPutAll: 'Errors:'; cr;
        	nextPutAll: (self clean: self errors); nextPut: $"; cr; tab;
        	nextPutAll: 'self notImplementedYet'
! 
printReturnValuesOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies
	the receiver's parameterSpecification"

	aStream nextPutAll: 'Return Values:'; cr.
	self returnValueSpecifications
						do: [:each | aStream nextPutAll: each; cr]!  
readErrorsFrom: aStream in: aChapter
	" Private - Read the receiver's errors from aStream. "

	aChapter upToLine: 'Errors' in: aStream.
	self errors: (aStream upTo: nil).!   
hasParameters
	"Private - Answer true if the receiver has parameters"

	^(Message numberOfArgumentsFor: self selector) ~= 0!   
initializeReturnValueSpecifications
	" Private - Initializes the receiver's returnValueSpecifications"

	returnValueSpecifications := OrderedCollection new.!  
readSelectorFrom: aStream
	" Private - Read the receiver's selector from aStream. "

	| elements |
	message := aStream upTo: $:; countBlanks; upTo: nil.
	selector := (message includes: $:)
	ifTrue: [
		elements := String new.
		(message asArrayOfSubstrings
			select: [:each| each includes: $: ])
			do: [:one| elements := elements , one ].
		elements asSymbol
	]
	ifFalse: [ (message upTo: $ ) asSymbol ]!   
selector: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    selector := anObject! 
readSynopsisFrom: aStream in: aChapter
	" Private - Read the receiver's synopsis from aStream. "

	aChapter upToLine: 'Synopsis' in: aStream.
	self synopsis: (aChapter skipToLineStartingWith: 'Definition:' in: aStream).!  
readDefinitionFrom: aStream in: aChapter
	" Private - Read the receiver's definition from aStream. "

	self hasParameters
				ifTrue: [^self definition: (aChapter skipToLine: 'Parameters' in: aStream)].

	self definition: (aChapter skipToLineStartingWith: 'Return' in: aStream)!  
readParametersFrom: aStream in: aChapter
	" Private - Read the receiver's parameters from aStream. "
	
	| text |
	self hasParameters ifFalse: [ ^self ].
	aChapter upToLine: 'Parameters' in: aStream.
	text := ReadStream on: (aChapter skipToLineStartingWith: 'Return' in: aStream).
	[text atEnd] whileFalse: [
		self parameterSpecifications add: text nextLine
			"add: (ParameterSpecification from: (ReadStream on: text nextLine))"
	].!  
refinement: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    refinement := anObject! 
definition: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    definition := anObject! 
parameterSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	parameterSpecifications isNil ifTrue: [ self initializeParameterSpecifications ].
    ^parameterSpecifications!  
synopsis
    "Returns the instance var of the receiver
     for external manipulation"
    ^synopsis!  
errors: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    errors := anObject! 
errors
    "Returns the instance var of the receiver
     for external manipulation"
    ^errors!  
synopsis: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    synopsis := anObject! 
printParametersOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies
	the receiver's parameterSpecification"

	aStream nextPutAll: 'Parameters:'; cr.
	self parameterSpecifications
						do: [:each | aStream nextPutAll: each; cr]! 
readReturnValuesFrom: aStream in: aChapter
	" Private - Read the receiver's returnValue from aStream. "

	| text |
	#Gallego.
	aChapter upToLineStartingWith: 'Return' in: aStream.
	text := ReadStream on: (aChapter skipToLine: 'Errors' in: aStream).
	[text atEnd] whileFalse: [
		self returnValueSpecifications add: text nextLine
			"add: (ReturnValueSpecification from: (ReadStream on: text nextLine))"
	].! 
clean: aString
	"Answer a String without comment characters"

	^aString reject: [ :char | char = $" ]! 
returnValueSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	returnValueSpecifications isNil ifTrue: [ self initializeReturnValueSpecifications ].
    ^returnValueSpecifications!  
initializeParameterSpecifications
	" Private - Initializes the receiver's parameterSpecifications"

	parameterSpecifications := OrderedCollection new.!
selector
    "Returns the instance var of the receiver
     for external manipulation"
    ^selector!  
aliasing
    "Returns the instance var of the receiver
     for external manipulation"
    ^aliasing!  
protocols
    "Returns the instance var of the receiver
     for external manipulation"

	protocols isNil ifTrue: [ self initializeProtocols ].
    ^protocols!  
aliasing: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    aliasing := anObject! 
initializeProtocols
	" Private - Initializes the receiver's protocols"

	protocols := OrderedCollection new.!  
name
    "Returns the instance var of the receiver
     for external manipulation"
    ^name!  
name: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    name := anObject! 
description
    "Returns the instance var of the receiver
     for external manipulation"
    ^description!
description: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    description := anObject!   
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	aStream
		nextPutAll: self name; cr; cr;
		nextPutAll: self description!   
protocolFrom: aChapter
	" Private - Returns a protocol from aChapter. "

	^ProtocolSpecification from: aChapter!   
initialize: aChapter
    "Private - Initialize the receiver."

	| stream |
	stream := ReadStream on: aChapter contents asArray.
    self
		name: aChapter title;
		description: stream next contents contents;
		yourself.
	[stream atEnd] whileFalse: [
		self protocols add: (self protocolFrom: stream next)
	].!   
initializeProtocols
	" Private - Initializes the receiver's protocols"

	protocols := OrderedCollection new.!  
protocols
    "Returns the instance var of the receiver
     for external manipulation"

	protocols isNil ifTrue: [ self initializeProtocols ].
    ^protocols!  
name: aString
    "Return a new instance of the receiver named aString"

     ^self new
        name: aString;
        yourself.!
name
    "Returns the name of the receiver."

    ^name
! 
name: aString
    "Sets the name of the receiver."

    name := aString

!   
printOn: aStream
	"Append to the argument, aStream, a sequence of characters that identifies the receiver."

	aStream
		nextPutAll: 'Protocol: <';
		nextPutAll: self name;
		nextPut: $>;cr;cr;
		nextPutAll: 'Conforms To:'; cr.
	self conformsTo
		do: [ :each | each printOn: aStream	].
	aStream
		cr;cr;
		nextPutAll: self description!
readContentsFrom: aStream in: aChapter
	" Private - Read the receiver's contents from aStream. "

	self
		readConformsFrom: aStream in: aChapter;
		readDescriptionFrom: aStream in: aChapter;
		yourself!  
conformsTo: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    conformsTo := anObject! 
initialize: aChapter
	" Private - Initialize the receiver. "

	| stream |
	stream := ReadStream on: aChapter title.
    self name: (stream upTo: $<;upTo: $>).
	stream := ReadStream on: aChapter contents asArray.
	self
		readContentsFrom: (ReadStream on: stream next text)
		in: aChapter.
	[stream atEnd] whileFalse: [
		self messageSpecifications add: (self messageSpecificationFrom: stream next)
	].! 
readConformsFrom: aStream in: aChapter
	" Private - Read the receiver's conformsTo from aStream. "

	| conforms |
	aChapter upToLine: 'Conforms To' in: aStream.
	conforms := ReadStream on: (aChapter skipToLine: 'Description' in: aStream).
	conformsTo := Set new.
	[conforms atEnd] whileFalse: [
		conformsTo add: (conforms upTo: $,) trimBlanks
	].! 
conformsTo
    "Returns the instance var of the receiver
     for external manipulation"
    ^conformsTo!  
readDescriptionFrom: aStream in: aChapter
	" Private - Read the receiver's description from aStream. "

	self description: (aStream upTo: nil)!
messageSpecificationFrom: aChapter
	" Private - Returns the next messageSpecification from aChapter. "

	^MessageSpecification from: aChapter! 
compileInClass: aClass
	"Compile all the receiver's messageSpecifications in aClass"

	| assoc |
	self messageSpecifications do: [:msg |
														assoc := aClass compile: msg printString notifying: Transcript.
														assoc notNil
																	ifTrue: [SourceManager current
																					logSource: msg printString
							        												forSelector: assoc key
							        												inClass: aClass ]
														]!  
description
    "Returns the description of the receiver."

    ^description
!
description: aString
    "Sets the description of the receiver."

    description := aString

!  
messageSpecifications
    "Returns the instance var of the receiver
     for external manipulation"

	messageSpecifications isNil ifTrue: [ self initializeMessageSpecifications ].
    ^messageSpecifications!  
initializeMessageSpecifications
	" Private - Initializes the receiver's messageSpecifications"

	messageSpecifications := OrderedCollection new!   
defaultGroupIDs
	" Private - Returns the IDs to be used to read the receiver's groups. "

	^(3 to: 10) collect: [:each| '5.',each asString ]!  
initializeProtocolGroups
	" Private - Initializes the receiver's protocolGroups"

	protocolGroups := OrderedCollection new.!   
groups:  ids from: aDocument
	" Private - Returns the groups read from aDocument with ids. "

	^ids collect: [:id| ProtocolGroupSpecification from: aDocument @ id ]!  
protocolNamed: aName ifAbsent: aBlock
	"Return the protocol named aName or evaluates aBlock if absent."

	| protocol |
	self protocolGroups do: [:each|
		protocol := each protocols
			detect: [:one| one name = aName ]
			ifNone: [].
		protocol notNil ifTrue: [ ^protocol ].
	].
	^aBlock value!   
protocolGroups
    "Returns the instance var of the receiver
     for external manipulation"

	protocolGroups isNil ifTrue: [ self initializeProtocolGroups ].
    ^protocolGroups!  
initialize: aDocument
	" Private - Initialize the receiver. "

	self protocolGroups addAll: (self groups: self defaultGroupIDs from: aDocument)!   
protocolNamed: aName
	"Return the protocol named aName or generates an error if absent."

	^self protocolNamed: aName ifAbsent: [ self error: 'Element not found.' ]!  
name
    "Returns the instance var of the receiver
     for external manipulation"
    ^name!  
name: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    name := anObject!