  
"
    __________________________________________________
    @Alejandro F. Reimondo 18/10/02
    This proyect sources are property of : Alejandro F. Reimondo
    Its use is NOT legal without written rights.
    Amenabar 3476 Piso: 2 Dto: B.
    Ciudad de Buenos Aires.
    Argentina.
    Tel: (54)-1-703-5586.
    __________________________________________________
"!

Object subclass: #NCITS319Object
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''    !

NCITS319Object subclass: #NCITSDocumentElement
  instanceVariableNames: 
    ' parent '
  classVariableNames: ''
  poolDictionaries: ''    !

NCITSDocumentElement subclass: #NCITSChapter
  instanceVariableNames: 
    ' contents '
  classVariableNames: ''
  poolDictionaries: ''    !

NCITSChapter subclass: #NCITSCompositeChapter
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''   !

NCITSChapter subclass: #NCITSSimpleChapter
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

NCITSDocumentElement subclass: #NCITSChapterReference
  instanceVariableNames: 
    ' title id pageNumber contents '
  classVariableNames: ''
  poolDictionaries: ''   !

NCITSDocumentElement subclass: #NCITSDocument
  instanceVariableNames: 
    ' contents '
  classVariableNames: ''
  poolDictionaries: ''   !

NCITSDocument class instanceVariableNames: ' default ' !

NCITSDocumentElement subclass: #NCITSTableOfContents
  instanceVariableNames: 
    ' contents '
  classVariableNames: ''
  poolDictionaries: ''    !

NCITS319Object subclass: #TextStreamProcessor
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #TextStreamProcessor) comment: '
	Private - TextProcessors are used to remove obsolete text and do some text formating operations.' !

TextStreamProcessor subclass: #CompositeTextProcessor
  instanceVariableNames: 
    ' elements '
  classVariableNames: ''
  poolDictionaries: ''   !

TextStreamProcessor subclass: #SimpleTextProcessor
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''  !

SimpleTextProcessor subclass: #ChapterBreakProcessor
  instanceVariableNames: 
    ' root '
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #ChapterBreakProcessor) comment: '
	Try to find chapter breaks from in text.'    !

SimpleTextProcessor subclass: #PageBreakProcessor
  instanceVariableNames: ''
  classVariableNames: ''
  poolDictionaries: ''.

  (Smalltalk at: #PageBreakProcessor) comment: '
	Remove page breaks from text.' !


!NCITS319Object methods !

backUpOver: previousObject in: aStream
	"Reposition the receiver to before the previous object. "

	aStream position: aStream position - (previousObject size max: 1)!
   
does: aString endWith: token
	"Returns true iff aString end with token. "

	token isNil ifTrue: [ ^false ].
	^aString size >= token size
	and: [ (aString copyFrom: aString size - token size + 1 to: aString size) = token ]!
 
does: aString startWith: token
	"Returns true iff aString start with token. "

	token isNil ifTrue: [ ^false ].
	^aString size >= token size
	and: [ (aString copyFrom: 1 to: token size) = token ]!
   
nextIntegerFrom: aStream
	"Returns the next Integer read from aStream."

	^aStream nextInteger!
  
nextLineFrom: aStream
	"Returns the next line read from aStream."

	^self nextLineIn: aStream!
   
nextLineIn: aStream
	"Returns the next line read from aStream."

	^aStream nextLine!
 
skipToLine: contents in: aStream
	"Returns the contents in aStream upTo the line with 'contents'.
	Do NOT advance the stream over the contents. "

	^self skipToLineMatching: [:line| contents = line ] in: aStream!

skipToLineMatching: aBlock in: aStream
	"Returns the contents in aStream upTo the line matching aBlock. "

	| result line start |
	result := WriteStream on: String new.
	[	start := aStream position.
		aBlock value: (line := aStream nextLine) ] whileFalse: [
		result nextPutAll: line; cr.
		aStream atEnd ifTrue: [ ^result contents ].
	].
	aStream position: start.
	^result contents!
  
skipToLineStartingWith: token in: aStream
	"Returns the contents in aStream upTo the line starting with token. "

	^self skipToLineMatching: [:line|
		self does: line startWith: token
		] in: aStream!
   
upTo: token in: aStream
	"Returns the contents of aStream upTo token. "

	| result test position |
	token isNil ifTrue: [ ^aStream upTo: nil ].
	token isCharacter ifTrue: [ ^aStream upTo: token ].
	result := String new.
	[	result := result , (aStream upTo: token first).
		aStream atEnd "...notFound..."
	]	whileFalse: [
		position := aStream position.
		aStream backupOver: token first.
		test := aStream next: token size.
		test = token ifTrue: [ ^result ].
		result := result, test.
		aStream position: position.
	].
	^result!
  
upToLine: contents in: aStream
	"Returns the contents in aStream upTo the line with 'contents'. "

	^self upToLineMatching: [:line| contents = line ] in: aStream!
   
upToLineMatching: aBlock in: aStream
	"Returns the contents in aStream upTo the line matching aBlock. "

	| result |
	result := self skipToLineMatching: aBlock in: aStream.
	aStream nextLine.
	^result!
 
upToLineStartingWith: token in: aStream
	"Returns the contents in aStream upTo the line strating with token. "

	| result |
	result := self skipToLineStartingWith: token in: aStream.
	aStream nextLine.
	^result! !



!NCITSDocumentElement class methods !

checks
	" Private - Returns the check for contents validation when reading instances.
	The default answer is an empty check collection.
	This method can be refined by the subclasses. "

	^#( )!
  
in: aParent
	"Returns an instance of the receiver contained in aParent."

	^self new initialize: aParent!

in: aParent readFrom: aStream
	"Returns an instance of the receiver read from aStream."

	^(self in: aParent) readFrom: aStream!
 
readFrom: aStream
	"Returns an instance of the receiver read from aStream."

	^self in: nil readFrom: aStream! !



!NCITSDocumentElement methods !
  
check: aStream
	" Private - Check the contents of aStream as a valid file. "

	| current |
	self checks do: [:token|
		current := self nextLineIn: aStream.
		(self does: current startWith: token) ifFalse: [
			^self error: 'Invalid contents.'
		]
	].
	^true!

checks
	" Private - Returns the check for contents validation when reading the receiver.
	The default answer is managed by the receiver's species. "

	^self species checks!

initialize: aParent
	" Private - Initialize the receiver. "

	self parent: aParent!
  
parent
    "Returns the instance var of the receiver
     for external manipulation"
    ^parent!

parent: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    parent := anObject!
   
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	^self implementedBySubclass "subclassResponsibility"!

readFrom: aStream
	" Private - Read the contents of the receiver from aStream. "

	self
		check: aStream;
		readContentsFrom: aStream;
		yourself.! !



!NCITSChapter class methods !

defaultProcessor
	" Private - Return the text processor to be used for the receiver's instances. "

	^TextStreamProcessor default!
   
defaultSupport
	" Private - Returns the default support for raw text chapters. "

	^NCITSSimpleChapter!
  
in: aChapterReference
	" Private - Returns a concrete instance of the receiver capable to handle the syntax of aChapterReference. "

	| support |
	support := self allSubclasses
		detect: [:one| one supports: aChapterReference ]
		ifNone: [ self defaultSupport ].
	^support new
		initialize: aChapterReference!
   
processor
	" Private - Return the text processor support to be used for the receiver's instances. "

	^self subclassResponsibility!
  
supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference.
	This method must be implemented by the subclasses. "

	^self implementedBySubclass "subclassResponsibility"! !



!NCITSChapter methods !
 
checks
	" Private - Returns the check for contents validation when reading the receiver."

	^Array new "with: self parent token"!

contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!

contents: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    contents := anObject!
   
id
	"Returns the chapter ID."

	^parent id!
  
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	super printOn: aStream.
	aStream
		nextPut: $[;
		nextPutAll: self parent description;
		nextPut: $];
		yourself!
 
processor
	" Private - Return the text processor support to be used for the receiver. "

	^self species processor!
   
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	self contents: (self processor value: aStream).!
 
text
	"Returns the chapter text."

	^String new!
 
title
	"Returns the chapter title."

	^parent title!
 
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^(aBlock value: self)
		ifTrue: [ self ]
		ifFalse: [ absentBlock value ]!
  
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^aBlock value: (selectBlock value: self)! !



!NCITSCompositeChapter class methods !
   
processor
	" Private - Return the text processor to be used for the receiver's instances. "

	^self defaultProcessor!

supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference. "

	^aChapterReference id notNil! !



!NCITSCompositeChapter methods !
 
asArray
	"Returns the receiver as an array."

	^self contents asArray!
   
first
	"Returns the first element."

	^self contents first!
  
processor
	" Private - Return the text processor support to be used for the receiver. "

	^self species processor
	,(ChapterBreakProcessor new
		root: self parent;
		yourself)!
  
readFrom: aStream
	" Private - Read the contents of the receiver from aStream. "

	super readFrom: aStream.
	self contents size = 1 ifTrue: [ "really not a composite"
		^self contents first contents
	].!
   
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	| result |
	self contents do: [:each|
		result := each whoContains: aBlock ifAbsent: [].
		result notNil ifTrue: [ ^result ].
	].
	^super whoContains: aBlock ifAbsent: absentBlock!
 
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	self contents do: [:each|
		aBlock value: (selectBlock value: each)
	].
	^super with: selectBlock do: aBlock! !



!NCITSSimpleChapter class methods !
   
processor
	" Private - Return the text processor to be used for the receiver's instances. "

	^self defaultProcessor!

supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference. "

	^false "default support"! !



!NCITSSimpleChapter methods !

asArray
	"Returns the receiver as an array."

	^Array with: self!

text
	"Returns the receiver's text."

	^self contents! !



!NCITSChapterReference class methods !
   
forText: aString
	"Returns an instance of the receiver containing text aString."

	^self new
		readContentsFrom: (ReadStream on: aString) upTo: nil!

validTitle: aString
	" Private - Correct wrong titles. "

	aString = 'The syntax of Smalltalk Programs' ifTrue: [
		^'The Syntax of Smalltalk Programs'
	].
	^aString! !



!NCITSChapterReference methods !
  
contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!

contents: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    contents := anObject!
   
description
	" Private - Returns the normalized representation of the receiver. "

	| tag |
	tag := self id isNil
		ifTrue: [ String new ]
		ifFalse: [ self id asString, ' ' ].
	^tag, self title asString
!
   
id
    "Returns the instance var of the receiver
     for external manipulation"
    ^id!

id: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    id := anObject!
   
initializeTitle
	" Private - Initializes the receiver's title"

	title := ' '.!
  
isEmpty
	"Returns true if the receiver is empty."

	^contents isNil!
 
mainToken
	" Private - Returns the normalized representation of the receiver used for contents. "

	| tag |
	tag := self id isNil
		ifTrue: [ String new ]
		ifFalse: [
			self id asString,
				((self id asString includes: $.)
				ifTrue: [' '] ifFalse: ['. '])
		].
	^tag, self title asString
!
  
pageNumber
    "Returns the instance var of the receiver
     for external manipulation"
    ^pageNumber!

pageNumber: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    pageNumber := anObject!
   
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	aStream	cr;
		nextPut: $#; nextPutAll: self description!
  
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	| source |
	source := ReadStream on: (self nextLineFrom: aStream).
	source peek isDigit ifTrue: [ self readIDFrom: source ].
	self
		readTitleFrom: source;
		readPageNumberFrom: source;
		yourself.!
 
readContentsFrom: aStream upTo: token
	"Read the receiver's contents from aStream, prior to token."

	| source |
	source := ReadStream on: (self skipToLineStartingWith: token in: aStream).
	self contents: (NCITSChapter in: self readFrom: source).!

readIDFrom: aStream
	" Private - Read the ID of the receiver from aStream. "

	self id: (self upTo: $  in: aStream).!

readPageNumberFrom: aStream
	" Private - Read the pageNumber of the receiver from aStream. "

	self upTo: '... ' in: aStream.
	aStream atEnd ifTrue: [ ^self ].
	self pageNumber: (self nextIntegerFrom: aStream)!
 
readTitleFrom: aStream
	" Private - Read the title of the receiver from aStream. "

	| aTitle |
	aTitle := (self upTo: '...' in: aStream) trimBlanks.
	self title: (self species validTitle: aTitle).!
 
text
	"Returns the receiver's text."

	^self contents contents!
  
title
    "Returns the instance var of the receiver
     for external manipulation"

	title isNil ifTrue: [ self initializeTitle ].
    ^title!

title: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    title := anObject!
 
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^self isEmpty ifFalse: [
		self contents whoContains: aBlock ifAbsent: absentBlock
	]!
  
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^self isEmpty ifFalse: [
		aBlock value: (selectBlock value: self contents)
	]! !



!NCITSDocument class methods !
   
checks
	" Private - Returns the check for contents validation when reading instances. "

	^super checks,
	#(
		'American National Standards Institute'
		'11 West 42nd Street'
		'New York, New York'
		'10036'
		'ANSI NCITS 319-1998'
	)!
   
default
    "Returns the instance var of the receiver
     for external manipulation"

	default isNil ifTrue: [ self initializeDefault ].
    ^default!

default: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    default := anObject!
 
fileIn: fileName
	"Read the contents of the file @ fileName and install its contents."

	| file |
	file := File pathNameReadOnly: fileName.
	[ self install: file ] ensure: [ file close ].!
   
initializeDefault
	" Private - Initializes the receiver's default"

	default := self new.!
   
install: aStream
	"Parse the contents of aStream and install it as the default instance."

	self default: (self readFrom: aStream)!
  
reset
	"Reset the receiver."

	self default: nil.! !



!NCITSDocument methods !
 
? mask
	" Returns the element with title matching mask (or nil). "

	^self whoContains: [:chapter| (chapter title indexOfPattern: mask) > 0 ]!
   
@ id
	" Returns the element with chapter id (or nil). "

	^self whoContains: [:chapter| chapter id = id ]!
   
contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!

find: mask
	" Returns the element with contents matching mask (or nil). "

	^self whoContains: [:chapter| (chapter text indexOfPattern: mask) > 0 ]!
 
readChaptersFrom: aStream
	" Private - Read the chapter contents of the receiver from aStream. "

	| entries |
	entries := ReadStream on: contents entries.
	[entries atEnd] whileFalse: [
		entries next
			readContentsFrom: aStream
			upTo: (entries atEnd
				ifFalse: [ entries peek mainToken ]).
		aStream nextLine "halt".
		(aStream atEnd and: [ entries atEnd not ]) ifTrue: [
			self error: 'Unspected EndOfFile.'
		].
	].!

readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	self skipToLine: 'Contents' in: aStream.
	contents := NCITSTableOfContents in: self readFrom: aStream.
	self readChaptersFrom: aStream.!
   
whoContains: aBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true (or nil). "

	^self whoContains: aBlock ifAbsent: [ ]!
   
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^self contents whoContains: aBlock ifAbsent: absentBlock!
 
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^self contents with: selectBlock do: aBlock!

withIDsDo: aBlock
	" Evaluates aBlock for all IDs. "

	^self with: [:chapter| chapter id ] do: aBlock!
   
withTextsDo: aBlock
	" Evaluates aBlock for all texts. "

	^self with: [:chapter| chapter text ] do: aBlock!
 
withTitlesDo: aBlock
	" Evaluates aBlock for all titles. "

	^self with: [:chapter| chapter title ] do: aBlock! !



!NCITSTableOfContents class methods !
   
checks
	" Private - Returns the check for contents validation when reading instances. "

	^super checks,
	#(
		'Contents'
		'Page'
	)! !



!NCITSTableOfContents methods !
  
contents
    "Returns the instance var of the receiver
     for external manipulation"

	contents isNil ifTrue: [ self initializeContents ].
    ^contents!

description
	" Private - Returns the description of the receiver's entries."

	| aStream |
	aStream := WriteStream on: String new.
	self entries do: [:each|
		aStream cr; nextPutAll: each description.
	].
	^aStream contents!

entries
    "Returns the entries of the receiver."

	^self contents!
 
initializeContents
	" Private - Initializes the receiver's contents"

	contents := OrderedCollection new.!
   
nextItemFrom: aStream
	" Private - Return the next item read from aStream (or nil). "

	| position chunk title |
	position := aStream position.
	chunk := self nextLineIn: aStream.
	self contents notEmpty ifTrue: [
		title := self contents first description.
		(self does: chunk startWith: title)
		ifTrue: [ aStream position: position. ^nil ].
	].

	^NCITSChapterReference in: self readFrom: chunk asStream!
 
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	super printOn: aStream.
	aStream
		nextPut: $[;
		nextPutAll: self description;
		nextPut: $];
		yourself!

readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	| item |
	[	item := self nextItemFrom: aStream.
		item notNil
	] whileTrue: [ self contents add: item ].!
 
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	| result |
	self contents do: [:each|
		result := each whoContains: aBlock ifAbsent: [].
		result notNil ifTrue: [ ^result ].
	].
	^absentBlock value!
   
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	| result |
	self contents do: [:each|
		result := aBlock value: (selectBlock value: each).
		each with: selectBlock do: aBlock.
	].
	^result! !



!TextStreamProcessor class methods !

compositeSupport
	" Private - Return the compositeSupport for the receiver's instances. "

	^CompositeTextProcessor!
 
default
	"Returns the default instance of the receiver."

	^PageBreakProcessor new

	"^self empty" "do not transform source contents"!
 
empty
	"Returns an empty instance of the receiver.
	This instance will leave the source unmodified."

	^self compositeSupport empty! !



!TextStreamProcessor methods !

, aProcessor
	"Returns a composite processor with the receiver and aProcessor."

	^self compositeSupport with: self with: aProcessor!

compositeSupport
	" Private - Retrns the composite support for the receiver. "

	^self species compositeSupport!
 
outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	^WriteStream on: (String new: aStream size)!

process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	^self implementedBySubclass!
 
value: aStream
	"Returns the result of evaluating the receiver on aStream."

	| output |
	output := self outputFor: aStream.
	self process: aStream on: output.
	^output contents! !



!CompositeTextProcessor class methods !
   
empty
	"Returns an empty instance of the receiver.
	This instance will leave the source unmodified."

	^self new!
   
with: aProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		yourself!
 
with: aProcessor
with: bProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		yourself!
   
with: aProcessor
with: bProcessor
with: cProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		add: cProcessor;
		yourself!
 
with: aProcessor
with: bProcessor
with: cProcessor
with: dProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		add: cProcessor;
		add: dProcessor;
		yourself! !



!CompositeTextProcessor methods !

, aProcessor
	"Returns a composite processor with the receiver and aProcessor."

	self add: aProcessor.
	^self!
 
add: anElement
	" Private - Add anElement into the receiver's elements. "

	self elements add: anElement.
	"WARNING: it is important to be added this way to be used as wrappers."!
 
elements
    "Returns the instance var of the receiver
     for external manipulation"

	elements isNil ifTrue: [ self initializeElements ].
    ^elements!

initializeElements
	" Private - Initializes the receiver's elements"

	elements := OrderedCollection new.!
   
isEmpty
	"Returns true if the receiver is empty."

	^elements size = 0!
  
outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	self isEmpty ifFalse: [
		^self elements last outputFor: aStream
	].
	^super outputFor: aStream!
 
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| source destination |
	source := input.
	self elements do: [:each|
		destination := each outputFor: source.
		destination nextPutAll: (each value: source).
		source := ReadStream on: destination contents.
	].
	output nextPutAll: destination contents.! !



!ChapterBreakProcessor methods !

outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	^WriteStream on: (Array new: 100)!
  
placeTextWithoutChapters: text on: output
	" Private - Place a raw text on output. "

	text size = 0 ifTrue: [ ^self ].
	output nextPut: (NCITSChapterReference forText: text).!

process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| chapter text current id chunk |
	current := 1.
	id := self tagFor: current.
	text := self untaggedText: id in: input.
	self placeTextWithoutChapters: text on: output.
	[ input atEnd ] whileFalse: [
		chunk := self nextLineIn: input.
		input peek isLowercase ifTrue: [chunk := chunk , ' ', (self nextLineIn: input)].
		chapter := (NCITSChapterReference in: self root) readFrom: chunk asStream.
		chapter readContentsFrom: input upTo: (self tagFor: current + 1).
		output nextPut: chapter.
		current := current + 1.
		id := self tagFor: current.
	].!
  
root
    "Returns the instance var of the receiver
     for external manipulation"
    ^root!

root: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    root := anObject!
   
tagFor: aNumber
	" Private - Returns the tag to be used to detect the sub-chapter aNumber. "

	^self root id asString,'.',aNumber asString,' '!
  
taggedText: token in: aStream
	" Private - Scan the input for text with chapter header and returns the raw text result. "
self halt.
	^self skipToLineStartingWith: token in: aStream!
   
untaggedText: token in: aStream
	" Private - Scan the input for text without chapter header and returns the raw text result. "

	^self skipToLineStartingWith: token in: aStream! !



!PageBreakProcessor methods !
 
pageBreakToken
	" Private - Return the token to be used to detect page breaks. "

	^'.ANSI NCITS 319-1998'!
  
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| line |
	[input atEnd] whileFalse: [
		line := self nextLineIn: input.
		(self does: line endWith: self pageBreakToken)
		ifTrue: [ self processPageBreakIn: line from: input to: output ]
		ifFalse: [ output nextPutAll: line; cr ]
	].!

processPageBreakIn: line from: input to: output
	" Private - Try to execute a pageBreak remotion. "

	| position pageNumber |
	position := input position.
	pageNumber := self nextIntegerFrom: (ReadStream on: (self nextLineIn: input)).
	pageNumber = 0 ifTrue: [ "not a pageBreak."
		input position: position.
		output nextPutAll: line.
		^self
	].
	output
		nextPutAll: (line copyFrom: 1 to: line size - self pageBreakToken size);
		cr.! !
