18/10/02 04:38:14 p.m.
  
checks
	" Private - Returns the check for contents validation when reading instances. "

	^super checks,
	#(
		'American National Standards Institute'
		'11 West 42nd Street'
		'New York, New York'
		'10036'
		'ANSI NCITS 319-1998'
	)! 
default
    "Returns the instance var of the receiver
     for external manipulation"

	default isNil ifTrue: [ self initializeDefault ].
    ^default!  
default: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    default := anObject!   
initializeDefault
	" Private - Initializes the receiver's default"

	default := self new.! 
install: aStream
	"Parse the contents of aStream and install it as the default instance."

	self default: (self readFrom: aStream)!
fileIn: fileName
	"Read the contents of the file @ fileName and install its contents."

	| file |
	file := File pathNameReadOnly: fileName.
	[ self install: file ] ensure: [ file close ].! 
reset
	"Reset the receiver."

	self default: nil.! 
in: aParent
	"Returns an instance of the receiver contained in aParent."

	^self new initialize: aParent!  
readFrom: aStream
	"Returns an instance of the receiver read from aStream."

	^self in: nil readFrom: aStream! 
in: aParent readFrom: aStream
	"Returns an instance of the receiver read from aStream."

	^(self in: aParent) readFrom: aStream!   
checks
	" Private - Returns the check for contents validation when reading instances.
	The default answer is an empty check collection.
	This method can be refined by the subclasses. "

	^#( )!
default
	"Returns the default instance of the receiver."

	^PageBreakProcessor new

	"^self empty" "do not transform source contents"!   
compositeSupport
	" Private - Return the compositeSupport for the receiver's instances. "

	^CompositeTextProcessor!   
empty
	"Returns an empty instance of the receiver.
	This instance will leave the source unmodified."

	^self compositeSupport empty!  
skipToLine: contents in: aStream
	"Returns the contents in aStream upTo the line with 'contents'.
	Do NOT advance the stream over the contents. "

	^self skipToLineMatching: [:line| contents = line ] in: aStream!  
nextLineFrom: aStream
	"Returns the next line read from aStream."

	^self nextLineIn: aStream! 
skipToLineMatching: aBlock in: aStream
	"Returns the contents in aStream upTo the line matching aBlock. "

	| result line start |
	result := WriteStream on: String new.
	[	start := aStream position.
		aBlock value: (line := aStream nextLine) ] whileFalse: [
		result nextPutAll: line; cr.
		aStream atEnd ifTrue: [ ^result contents ].
	].
	aStream position: start.
	^result contents!
does: aString startWith: token
	"Returns true iff aString start with token. "

	token isNil ifTrue: [ ^false ].
	^aString size >= token size
	and: [ (aString copyFrom: 1 to: token size) = token ]! 
upTo: token in: aStream
	"Returns the contents of aStream upTo token. "

	| result test position |
	token isNil ifTrue: [ ^aStream upTo: nil ].
	token isCharacter ifTrue: [ ^aStream upTo: token ].
	result := String new.
	[	result := result , (aStream upTo: token first).
		aStream atEnd "...notFound..."
	]	whileFalse: [
		position := aStream position.
		aStream backupOver: token first.
		test := aStream next: token size.
		test = token ifTrue: [ ^result ].
		result := result, test.
		aStream position: position.
	].
	^result!
nextIntegerFrom: aStream
	"Returns the next Integer read from aStream."

	^aStream nextInteger!
skipToLineStartingWith: token in: aStream
	"Returns the contents in aStream upTo the line starting with token. "

	^self skipToLineMatching: [:line|
		self does: line startWith: token
		] in: aStream! 
upToLineMatching: aBlock in: aStream
	"Returns the contents in aStream upTo the line matching aBlock. "

	| result |
	result := self skipToLineMatching: aBlock in: aStream.
	aStream nextLine.
	^result!   
upToLineStartingWith: token in: aStream
	"Returns the contents in aStream upTo the line strating with token. "

	| result |
	result := self skipToLineStartingWith: token in: aStream.
	aStream nextLine.
	^result! 
nextLineIn: aStream
	"Returns the next line read from aStream."

	^aStream nextLine!   
upToLine: contents in: aStream
	"Returns the contents in aStream upTo the line with 'contents'. "

	^self upToLineMatching: [:line| contents = line ] in: aStream! 
does: aString endWith: token
	"Returns true iff aString end with token. "

	token isNil ifTrue: [ ^false ].
	^aString size >= token size
	and: [ (aString copyFrom: aString size - token size + 1 to: aString size) = token ]!   
backUpOver: previousObject in: aStream
	"Reposition the receiver to before the previous object. "

	aStream position: aStream position - (previousObject size max: 1)! 
pageBreakToken
	" Private - Return the token to be used to detect page breaks. "

	^'.ANSI NCITS 319-1998'!
processPageBreakIn: line from: input to: output
	" Private - Try to execute a pageBreak remotion. "

	| position pageNumber |
	position := input position.
	pageNumber := self nextIntegerFrom: (ReadStream on: (self nextLineIn: input)).
	pageNumber = 0 ifTrue: [ "not a pageBreak."
		input position: position.
		output nextPutAll: line.
		^self
	].
	output
		nextPutAll: (line copyFrom: 1 to: line size - self pageBreakToken size);
		cr.! 
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| line |
	[input atEnd] whileFalse: [
		line := self nextLineIn: input.
		(self does: line endWith: self pageBreakToken)
		ifTrue: [ self processPageBreakIn: line from: input to: output ]
		ifFalse: [ output nextPutAll: line; cr ]
	].!  
outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	^WriteStream on: (String new: aStream size)!  
compositeSupport
	" Private - Retrns the composite support for the receiver. "

	^self species compositeSupport!   
value: aStream
	"Returns the result of evaluating the receiver on aStream."

	| output |
	output := self outputFor: aStream.
	self process: aStream on: output.
	^output contents!  
, aProcessor
	"Returns a composite processor with the receiver and aProcessor."

	^self compositeSupport with: self with: aProcessor!  
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	^self implementedBySubclass!   
processor
	" Private - Return the text processor to be used for the receiver's instances. "

	^self defaultProcessor!  
supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference. "

	^aChapterReference id notNil! 
in: aChapterReference
	" Private - Returns a concrete instance of the receiver capable to handle the syntax of aChapterReference. "

	| support |
	support := self allSubclasses
		detect: [:one| one supports: aChapterReference ]
		ifNone: [ self defaultSupport ].
	^support new
		initialize: aChapterReference! 
defaultProcessor
	" Private - Return the text processor to be used for the receiver's instances. "

	^TextStreamProcessor default! 
defaultSupport
	" Private - Returns the default support for raw text chapters. "

	^NCITSSimpleChapter!
processor
	" Private - Return the text processor support to be used for the receiver's instances. "

	^self subclassResponsibility!
supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference.
	This method must be implemented by the subclasses. "

	^self implementedBySubclass "subclassResponsibility"!
outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	^WriteStream on: (Array new: 100)!
taggedText: token in: aStream
	" Private - Scan the input for text with chapter header and returns the raw text result. "
self halt.
	^self skipToLineStartingWith: token in: aStream! 
untaggedText: token in: aStream
	" Private - Scan the input for text without chapter header and returns the raw text result. "

	^self skipToLineStartingWith: token in: aStream!  
tagFor: aNumber
	" Private - Returns the tag to be used to detect the sub-chapter aNumber. "

	^self root id asString,'.',aNumber asString,' '!
placeTextWithoutChapters: text on: output
	" Private - Place a raw text on output. "

	text size = 0 ifTrue: [ ^self ].
	output nextPut: (NCITSChapterReference forText: text).!  
root: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    root := anObject! 
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| chapter text current id chunk |
	current := 1.
	id := self tagFor: current.
	text := self untaggedText: id in: input.
	self placeTextWithoutChapters: text on: output.
	[ input atEnd ] whileFalse: [
		chunk := self nextLineIn: input.
		input peek isLowercase ifTrue: [chunk := chunk , ' ', (self nextLineIn: input)].
		chapter := (NCITSChapterReference in: self root) readFrom: chunk asStream.
		chapter readContentsFrom: input upTo: (self tagFor: current + 1).
		output nextPut: chapter.
		current := current + 1.
		id := self tagFor: current.
	].!
root
    "Returns the instance var of the receiver
     for external manipulation"
    ^root!  
with: aProcessor
with: bProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		yourself! 
with: aProcessor
with: bProcessor
with: cProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		add: cProcessor;
		yourself!   
with: aProcessor
with: bProcessor
with: cProcessor
with: dProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		add: bProcessor;
		add: cProcessor;
		add: dProcessor;
		yourself! 
empty
	"Returns an empty instance of the receiver.
	This instance will leave the source unmodified."

	^self new! 
with: aProcessor
	"Return an instance of the receiver."

	^self new
		add: aProcessor;
		yourself!   
add: anElement
	" Private - Add anElement into the receiver's elements. "

	self elements add: anElement.
	"WARNING: it is important to be added this way to be used as wrappers."!   
initializeElements
	" Private - Initializes the receiver's elements"

	elements := OrderedCollection new.! 
outputFor: aStream
	" Private - Returns a stream to be used as output for input aStream. "

	self isEmpty ifFalse: [
		^self elements last outputFor: aStream
	].
	^super outputFor: aStream!   
elements
    "Returns the instance var of the receiver
     for external manipulation"

	elements isNil ifTrue: [ self initializeElements ].
    ^elements!  
, aProcessor
	"Returns a composite processor with the receiver and aProcessor."

	self add: aProcessor.
	^self!   
isEmpty
	"Returns true if the receiver is empty."

	^elements size = 0!
process: input on: output
	" Private - Scan the input stream and perform transformations writing the results on output. "

	| source destination |
	source := input.
	self elements do: [:each|
		destination := each outputFor: source.
		destination nextPutAll: (each value: source).
		source := ReadStream on: destination contents.
	].
	output nextPutAll: destination contents.!
@ id
	" Returns the element with chapter id (or nil). "

	^self whoContains: [:chapter| chapter id = id ]! 
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^self contents with: selectBlock do: aBlock!  
withIDsDo: aBlock
	" Evaluates aBlock for all IDs. "

	^self with: [:chapter| chapter id ] do: aBlock! 
whoContains: aBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true (or nil). "

	^self whoContains: aBlock ifAbsent: [ ]! 
readChaptersFrom: aStream
	" Private - Read the chapter contents of the receiver from aStream. "

	| entries |
	entries := ReadStream on: contents entries.
	[entries atEnd] whileFalse: [
		entries next
			readContentsFrom: aStream
			upTo: (entries atEnd
				ifFalse: [ entries peek mainToken ]).
		aStream nextLine "halt".
		(aStream atEnd and: [ entries atEnd not ]) ifTrue: [
			self error: 'Unspected EndOfFile.'
		].
	].!  
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^self contents whoContains: aBlock ifAbsent: absentBlock!   
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	self skipToLine: 'Contents' in: aStream.
	contents := NCITSTableOfContents in: self readFrom: aStream.
	self readChaptersFrom: aStream.! 
withTitlesDo: aBlock
	" Evaluates aBlock for all titles. "

	^self with: [:chapter| chapter title ] do: aBlock!
? mask
	" Returns the element with title matching mask (or nil). "

	^self whoContains: [:chapter| (chapter title indexOfPattern: mask) > 0 ]! 
contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!  
withTextsDo: aBlock
	" Evaluates aBlock for all texts. "

	^self with: [:chapter| chapter text ] do: aBlock!   
find: mask
	" Returns the element with contents matching mask (or nil). "

	^self whoContains: [:chapter| (chapter text indexOfPattern: mask) > 0 ]!   
parent: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    parent := anObject! 
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	^self implementedBySubclass "subclassResponsibility"!  
checks
	" Private - Returns the check for contents validation when reading the receiver.
	The default answer is managed by the receiver's species. "

	^self species checks!  
check: aStream
	" Private - Check the contents of aStream as a valid file. "

	| current |
	self checks do: [:token|
		current := self nextLineIn: aStream.
		(self does: current startWith: token) ifFalse: [
			^self error: 'Invalid contents.'
		]
	].
	^true!  
parent
    "Returns the instance var of the receiver
     for external manipulation"
    ^parent!  
readFrom: aStream
	" Private - Read the contents of the receiver from aStream. "

	self
		check: aStream;
		readContentsFrom: aStream;
		yourself.! 
initialize: aParent
	" Private - Initialize the receiver. "

	self parent: aParent!
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	| result |
	self contents do: [:each|
		result := each whoContains: aBlock ifAbsent: [].
		result notNil ifTrue: [ ^result ].
	].
	^super whoContains: aBlock ifAbsent: absentBlock!   
processor
	" Private - Return the text processor support to be used for the receiver. "

	^self species processor
	,(ChapterBreakProcessor new
		root: self parent;
		yourself)!
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	self contents do: [:each|
		aBlock value: (selectBlock value: each)
	].
	^super with: selectBlock do: aBlock!  
asArray
	"Returns the receiver as an array."

	^self contents asArray! 
readFrom: aStream
	" Private - Read the contents of the receiver from aStream. "

	super readFrom: aStream.
	self contents size = 1 ifTrue: [ "really not a composite"
		^self contents first contents
	].! 
first
	"Returns the first element."

	^self contents first!
contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!  
processor
	" Private - Return the text processor support to be used for the receiver. "

	^self species processor! 
title
	"Returns the chapter title."

	^parent title!   
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	self contents: (self processor value: aStream).!   
contents: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    contents := anObject! 
checks
	" Private - Returns the check for contents validation when reading the receiver."

	^Array new "with: self parent token"!  
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	super printOn: aStream.
	aStream
		nextPut: $[;
		nextPutAll: self parent description;
		nextPut: $];
		yourself!   
text
	"Returns the chapter text."

	^String new!   
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^(aBlock value: self)
		ifTrue: [ self ]
		ifFalse: [ absentBlock value ]!
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^aBlock value: (selectBlock value: self)! 
id
	"Returns the chapter ID."

	^parent id!
checks
	" Private - Returns the check for contents validation when reading instances. "

	^super checks,
	#(
		'Contents'
		'Page'
	)! 
initializeContents
	" Private - Initializes the receiver's contents"

	contents := OrderedCollection new.! 
nextItemFrom: aStream
	" Private - Return the next item read from aStream (or nil). "

	| position chunk title |
	position := aStream position.
	chunk := self nextLineIn: aStream.
	self contents notEmpty ifTrue: [
		title := self contents first description.
		(self does: chunk startWith: title)
		ifTrue: [ aStream position: position. ^nil ].
	].

	^NCITSChapterReference in: self readFrom: chunk asStream!   
contents
    "Returns the instance var of the receiver
     for external manipulation"

	contents isNil ifTrue: [ self initializeContents ].
    ^contents!  
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	| result |
	self contents do: [:each|
		result := each whoContains: aBlock ifAbsent: [].
		result notNil ifTrue: [ ^result ].
	].
	^absentBlock value! 
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	| item |
	[	item := self nextItemFrom: aStream.
		item notNil
	] whileTrue: [ self contents add: item ].!   
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	| result |
	self contents do: [:each|
		result := aBlock value: (selectBlock value: each).
		each with: selectBlock do: aBlock.
	].
	^result!
description
	" Private - Returns the description of the receiver's entries."

	| aStream |
	aStream := WriteStream on: String new.
	self entries do: [:each|
		aStream cr; nextPutAll: each description.
	].
	^aStream contents!  
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	super printOn: aStream.
	aStream
		nextPut: $[;
		nextPutAll: self description;
		nextPut: $];
		yourself!  
entries
    "Returns the entries of the receiver."

	^self contents!   
processor
	" Private - Return the text processor to be used for the receiver's instances. "

	^self defaultProcessor!  
supports: aChapterReference
	" Private - Must return true if the receiver's instances are capable to handle the syntax of aChapterReference. "

	^false "default support"! 
forText: aString
	"Returns an instance of the receiver containing text aString."

	^self new
		readContentsFrom: (ReadStream on: aString) upTo: nil!  
validTitle: aString
	" Private - Correct wrong titles. "

	aString = 'The syntax of Smalltalk Programs' ifTrue: [
		^'The Syntax of Smalltalk Programs'
	].
	^aString!  
text
	"Returns the receiver's text."

	^self contents! 
asArray
	"Returns the receiver as an array."

	^Array with: self!  
readIDFrom: aStream
	" Private - Read the ID of the receiver from aStream. "

	self id: (self upTo: $  in: aStream).!  
whoContains: aBlock ifAbsent: absentBlock
	" Private - Returns the element contained by the receiver who evaluates aBlock to true. "

	^self isEmpty ifFalse: [
		self contents whoContains: aBlock ifAbsent: absentBlock
	]!
pageNumber: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    pageNumber := anObject! 
readContentsFrom: aStream
	" Private - Read the internal contents of the receiver from aStream. "

	| source |
	source := ReadStream on: (self nextLineFrom: aStream).
	source peek isDigit ifTrue: [ self readIDFrom: source ].
	self
		readTitleFrom: source;
		readPageNumberFrom: source;
		yourself.!   
with: selectBlock do: aBlock
	" Private - Evaluates aBlock with all the result of evaluating selectBlock in the receiver's contents. "

	^self isEmpty ifFalse: [
		aBlock value: (selectBlock value: self contents)
	]! 
printOn: aStream
	"Append the ASCII representation of the receiver to aStream."

	aStream	cr;
		nextPut: $#; nextPutAll: self description!
text
	"Returns the receiver's text."

	^self contents contents!
readPageNumberFrom: aStream
	" Private - Read the pageNumber of the receiver from aStream. "

	self upTo: '... ' in: aStream.
	aStream atEnd ifTrue: [ ^self ].
	self pageNumber: (self nextIntegerFrom: aStream)!   
description
	" Private - Returns the normalized representation of the receiver. "

	| tag |
	tag := self id isNil
		ifTrue: [ String new ]
		ifFalse: [ self id asString, ' ' ].
	^tag, self title asString
! 
isEmpty
	"Returns true if the receiver is empty."

	^contents isNil!   
readTitleFrom: aStream
	" Private - Read the title of the receiver from aStream. "

	| aTitle |
	aTitle := (self upTo: '...' in: aStream) trimBlanks.
	self title: (self species validTitle: aTitle).!   
readContentsFrom: aStream upTo: token
	"Read the receiver's contents from aStream, prior to token."

	| source |
	source := ReadStream on: (self skipToLineStartingWith: token in: aStream).
	self contents: (NCITSChapter in: self readFrom: source).!  
title
    "Returns the instance var of the receiver
     for external manipulation"

	title isNil ifTrue: [ self initializeTitle ].
    ^title!  
title: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    title := anObject!   
contents
    "Returns the instance var of the receiver
     for external manipulation"
    ^contents!  
contents: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    contents := anObject! 
pageNumber
    "Returns the instance var of the receiver
     for external manipulation"
    ^pageNumber!  
id: anObject
    "Sets the instance var of the receiver
     with anObject. (No checking is done)"
    id := anObject! 
mainToken
	" Private - Returns the normalized representation of the receiver used for contents. "

	| tag |
	tag := self id isNil
		ifTrue: [ String new ]
		ifFalse: [
			self id asString,
				((self id asString includes: $.)
				ifTrue: [' '] ifFalse: ['. '])
		].
	^tag, self title asString
!
id
    "Returns the instance var of the receiver
     for external manipulation"
    ^id!  
initializeTitle
	" Private - Initializes the receiver's title"

	title := ' '.!